/**
    @addtogroup groupCore
    @{
    @file archdef.h
    @brief Зарезервированные строки описания аппаратной части.
    @details 

    <b>Подключение</b>: @verbatim #include <arch/archdef.h> @endverbatim
    
    @see Подробное описание в разделе \ref sectionArch.

    @}
 */


#ifndef _ARCHDEF_H_
#define _ARCHDEF_H_

/**
 *  @name Поля данных
 *  @anchor ancArchKeys
 *  @details Зарезервированные уникальные \b ключи для поиска
 *  параметров конфигурации проекта.
 *  @{
 */

/**
 *  @brief Название платы.
 *  @details Наименование целевой платы -- строковый параметр, 
 *  содержащий значение из группы \ref ahcArchBoardNames "макросов"
 *  известных названий плат. Для перечисленных плат большинство
 *  параметров конфигурации будет загружено при инициализации
 *  операционной системы.
 */
#define ARCH_BOARD "board-name"


/**
 *  @brief Название процессора.
 *  @details Название процессора -- строковый параметр, 
 *  содержащий значение из группы \ref ahcArchProcNames "макросов"
 *  имён процессоров.
 */
#define ARCH_CPU "cpu-name"

/**
 *  @brief Индикация работы системного таймера.
 *  @details Имя линии \b GPIO подключенной к светодиоду
 *  индикации работы системного таймера. Имя линии может
 *  может быть записано без пробелов, как
 *  в документации на процессор (\b PG1), либо в стиле @osName 
 *  (<b>P_G + 1</b>).
 *  @note Если данного светодиода нет на используемой плате,
 *  его можно указать с пустой строкой в качестве значения.
 */
#define ARCH_LED_SYSTEM "led-system"

/**
 *  @brief Индикация работы файловой системы.
 *  @details Имя линии \b GPIO подключенной к светодиоду
 *  индикации работы файловой системы. Имя линии может
 *  может быть записано без пробелов, как
 *  в документации на процессор (\b PG1), либо в стиле @osName 
 *  (<b>P_G + 1</b>).
 *  @note Если данного светодиода нет на используемой плате,
 *  его можно указать с пустой строкой в качестве значения.
 */
#define ARCH_LED_DISK "led-disk"


/**
 *  @brief ШИМ канал подсветки.
 *  @details Число -- номер канала ШИМ, 
 *  используемый для управления подсветкой дисплея.
 */
#define ARCH_BACKLIGHT_GPIO "backlight-pwm"


/**
 *  @brief Частота ШИМ канала подсветки.
 *  @details Число -- рекомендованная частота ШИМ сигнала в герцах, используемого
 *  для управления подсветкой дисплея. 
 *  Может варьироваться от установленного на плате аппаратного
 *  драйвера подсветки.
 */
#define ARCH_BACKLIGHT_FREQUENCY "backlight-freq"

/**
 *  @brief ШИМ управления светодиодной летной.
 *  @details Число -- номер канала ШИМ, 
 *  используемый для управления светодиодной подсветкой.
 *  Для управления используется однопроводная шина данных.
 */
#define ARCH_LEDLINE_PWM "ledline-pwm"

/**
 *  @brief Таймер управления светодиодной летной.
 *  @details Число -- номер номер таймера, 
 *  используемый для управления светодиодной подсветкой.
 *  Для управления используется однопроводная шина данных.
 */
#define ARCH_LEDLINE_TIMER "ledline-timer"


/// @}


/**
 *  @name Названия плат
 *  @anchor ahcArchBoardNames
 *  @details Зарезервированные строковые константы названий известных плат.
 *  @{
 */
#define ARCH_BOARD_SE8350_00 "SE8350-00"  ///< @details Плата светодиодной подсветки проекта "Cyclops".
#define ARCH_BOARD_SE8351_00 "SE8351-00"  ///< @details Плата проекта "Стериус".
#define ARCH_BOARD_SE_DB_A20_B254 "SE-DB-A20-B254"  ///< @details Отладочная плата модуля SE-SOM-A20.
/// @}


/**
 *  @name Названия процессоров
 *  @anchor ahcArchProcNames
 *  @details Зарезервированные строковые константы названий используемых процессоров.
 *  @{
 */
#define ARCH_PROC_V3S "V3s"  ///< @details Процессор \em Allwinner \b V3s.
#define ARCH_PROC_A20 "A20"  ///< @details Процессор \em Allwinner \b A20.
#define ARCH_PROC_A40 "A40"  ///< @details Процессор \em Allwinner \b A40.
#define ARCH_PROC_H3  "H3"   ///< @details Процессор \em Allwinner \b H3.
/// @}


#endif
