/*
     RTOS MULTEX KLOTHO

     Address Resolution Protocol
     
     Author: Sergey V.Sokolov
     
     05.2003
     
*/
#ifndef ARP_H_
#define ARP_H_

#ifdef __cplusplus
extern "C" {
#endif
#include "netdevice.h"

typedef struct arp_table_entry
               {
                unsigned int dev_idx;
				mac_addr_t mac_addr;
                ip_addr_t  ip_addr;
				mac_addr_t mac_self;
                ip_addr_t  ip_self;
               } ARP_ENTRY;

#define ARPHRD_ETHER  1 /* ethernet hardware address */                              

#define ARPOP_REQUEST 0x0100 /* request to resolve address */
#define ARPOP_REPLY   0x0200 /* response to previous request */


typedef struct arp_frame 
       {
          USHORT ar_hrd;  /* format of hardware address */
          USHORT ar_pro;  /* format of protocol address */
          UCHAR  ar_hln;  /* length of hardware address */
          UCHAR  ar_pln;  /* length of protocol address */
          USHORT ar_op;  /* one of: ARPOP_REQUEST, ARPOP_REPLY */
          mac_addr_t arp_sha; /* sender hardware address */
          ip_addr_t  arp_spa; /* sender protocol address */
          mac_addr_t arp_tha; /* target hardware address */
          ip_addr_t  arp_tpa; /* target protocol address */
       }ARP_FRM;

extern STATUS arpFind(ip_addr_t ip, mac_addr_t *mac, int *idx, 
               ip_addr_t *ip_s, mac_addr_t *mac_s);

extern STATUS arpProcessData(unsigned int idx, ETH_HDR *hdr, void *data, size_t dataLen);
               
STATUS arpResolve (ip_addr_t ip);


#ifdef __cplusplus
}
#endif
#endif // ifndef ARP_H_
