/*
Header file: CGI.h
*/
#ifndef CGI_H_
#define CGI_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>
#include <vector.h>

// Тип запроса
typedef enum {rtUNCNOWN=0,rtGET=1,rtPOST=2} REQUEST_TYPE;


typedef struct DataField 
         {
		  char *Name;
		  char *Value;
		  int vLen;
		  struct DataField *Next;
		 } TDATA_FIELD;
		 
typedef TDATA_FIELD *REQUEST_FIELD;
typedef TDATA_FIELD *CGI_VARIABLES;

typedef struct ActionProc
         {
		  char *Name;
		  int (*Function)();
		  struct ActionProc *Next;
		 }TACTION_PROC;
typedef TACTION_PROC *CGI_ACTIONS;		  
		  
typedef struct cgiRequest 
         {
		  REQUEST_TYPE  RType;
		  char *Action;
		  REQUEST_FIELD ContentFields;
		  REQUEST_FIELD QueryFields;
		  CGI_VARIABLES EnVars; // Переменные среды
		 }CGI_REQUEST;
		   		 
typedef struct cgiResponce 
         {
		  REQUEST_FIELD DataFields;
		  CGI_VARIABLES EnVars;
		 }CGI_RESPONCE; 
		 

typedef struct cgiTextBuffer
{
    size_t BufferWritePosition;
    char TextBuffer[2048 - sizeof(size_t)];
} CGI_TEXTBUFFER;

typedef struct cgiTextReturn
{
    sVector *ResponceDataVectorPtr;   // Vector with response data. Can be null. Contains CGI_TEXTBUFFER.

    struct
    {
        char *BufferPtr;
        int BufferSize;
    } SingleBuffer;
} CGI_TEXTRETURN;

typedef struct
         {
		  int fd;
		  int cgiEOF;
		  int cgiPage;       // Наличие незакрытого заголовка страницы
		  int cgiForm;       // Наличие незакрытого заголовка формы

		  CGI_TEXTRETURN TextReturn;

		  char *ContentBuf;     // Указатель на буфер с ответом
		  char *bhdr;           
		  
		  int   ContentBufSize;
		  int   ContentBufType;    // Тип 0:text/html 1:binary
		  
		  unsigned int SAdr;
		  
		  CGI_REQUEST  Request;
		  CGI_RESPONCE Responce;
		  CGI_VARIABLES Vars;
		  
		 } CGI_PROCESSOR;		  		 

typedef struct node
{
  char *Chain_Buf;     // Указатель на буфер с ответом
  int   Chain_BufSize;
  struct node *next;
}TChainElem;
// Установка обработчика по умолчанию
void cgiSetDefaultProc(int (*Procedure)(CGI_PROCESSOR*));
// Установка обработчика на акцию
void cgiSetActionProc(const char *Action, int (*Procedure)(CGI_PROCESSOR*));
// Разрешение доступа к ресурсу (файлу)
void enableURL(char *URL, char *ctype);
// Запуск сервера
void cgiProcessorStart(const char *DefaultPage);
// Запуск сервера без загрузки файловой коллекции
void cgiProcessorStartLocal(const char *DefaultPage);

// Процедуры, вызываемые внутри обработчика:

// Назначение текстового ответа HTML
void cgiSetResponceText(CGI_PROCESSOR *CP,const char *txt);
// Получение значения передаваемого параметра (строковое)
char *cgiGetParam(CGI_PROCESSOR *CP,const char *Name);
// Получение блока данных
char* cgiGetParamSz(CGI_PROCESSOR *CP, const char *Name, int *len);
// Получение значения передаваемого параметра (числовое)
int cgiGetIntParam(CGI_PROCESSOR *CP, const char *Name);

// Присвоение значения переменной
void cgiSetVar(CGI_PROCESSOR *CP, const char *Name, const char *Value);
// Форматирование переменной
void cgiFormatVar(CGI_PROCESSOR *CP, const char *Name, const char *format, ...);
// Присвоение целого числа
void cgiSetIntVar(CGI_PROCESSOR *CP, const char *Name, int Value);
// Определение типа запроса
REQUEST_TYPE cgiRequestType(CGI_PROCESSOR *CP);

// Прямая выдача буфера
void cgiAssignContent(CGI_PROCESSOR *CP, char *content, int length, int type);

// Прямая выдача четырёхзвенной цепочки (например циклического буфера)
void cgiAssignChain4(CGI_PROCESSOR *CP, char *p1, int l1, char *p2, int l2, char *p3, int l3, char *p4, int l4, int type);

// Стили текста

typedef enum
{
    tsLeft = 0x01,
    tsRight = 0x02,
    tsCenter = 0x04,
    tsBold = 0x08,
    tsItalic = 0x10
} textStyle;

// Создает стандартный заголовок HTML-страницы
void newPage(CGI_PROCESSOR *CP, const char *caption, ...);
// Создает заголовок HTML-страницы и помещает строку body в тег <body>
void newPageEx(CGI_PROCESSOR *CP, const char *caption, const char *body);
// Создает заголовок <form> Method=0: post; Method!=0: get
void newForm(CGI_PROCESSOR *CP, const char *Action, const int Method);
// Добавляет текстовую строку в HTML-документ
void htm(CGI_PROCESSOR *CP, const char *fmt, ...) __attribute__ ((format (printf, 2, 3)));
// Вставляет форматированную текстовую строку в HTML-документ
void htmText(CGI_PROCESSOR *CP, const char *txt, textStyle style, int font_size, const char *color, const char *font);
// Разрешает/запрещает замену переменных на их значения в тексте HTML-файла
// По умолчанию - разрешено.
void htmDecodeEnable();
void htmDecodeDisable();
#ifdef __cplusplus
}
#endif
#endif // ifndef CGI_H_
