/**
  * @addtogroup groupCore
  * @{
  * @file console.h
  * @brief Дополнительные функции для работы с текстовым вводом-выводом.
  * @details Некоторые функции, дополняющие стандартный функционал stdio.h.
  * @}
 */

#ifndef CONSOLE_H_
#define CONSOLE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "stdio.h"// gets, puts, putc

/**
 * Получить следующий символ из uart-консоли.
 * @return Символ.
 * @details Функция блокирует исполнение до тех пор, пока не получит символ.
 * @note Символ берется из uart-консоли, а не из stdin.
 */
char get_c(void);

/**
 * Распечатать целое число в stdout в десятичном формате.
 * @param N Целое число, которое будет выведено.
 */
void puti(int N);

/**
 * Вывести беззнаковый байт в stdout в шестнадцатеричном формате.
 * @param b Беззнаковый байт, который будет выведен.
 */
void putb(unsigned char b);

/**
 * Вывести беззнаковое 16-битное число в stdout в шестнадцатеричном формате.
 * @param w Беззнаковое 16-битное число, которое будет выведено.
 */
void putw(unsigned short w);

/**
 * Вывести беззнаковое 32-битное число в stdout в шестнадцатеричном формате.
 * @param l Беззнаковое 32-битное число, которое будет выведено.
 */
void putl(unsigned int l);

/**
 * Считать из uart-консоли беззнаковое число в шестнадцатеричном формате.
 * @details Число будет считываться пока не будет получен не-шестнадцатеричный символ.
 * @return Считанное число.
 */
unsigned int geth(void);

/**
 * Считать из строки беззнаковое число в шестнадцатеричном формате.
 * @details Число будет считываться пока не будет получен не-шестнадцатеричный символ.
 * @param str Указатель на строку.
 * @return Считанное число или 0, в случае ошибки.
 */
unsigned int strtoh(unsigned char *str);

#ifdef __cplusplus
}
#endif
#endif // ifndef CONSOLE_H_
