/**
    @addtogroup groupCore
    @{
    @file env_vars.h
    @brief Дополнительные функции для работы с переменными окружения.
    @details Некоторые функции, дополняющие стандартный функционал stdlib.h.
    @}
 */

#ifndef ENV_VARS_H_
#define ENV_VARS_H_

#ifdef __cplusplus
extern "C" {
#endif
/* Public declarations */

#include "stdlib.h" // getenv, setenv


struct env_var {
       char *name;
	   char *val;
	   struct env_var *next;
	   };

/**
 * Получить структуру env_var переменной окружения.
 * @param name Имя переменной окружения.
 * @return Указатель на структуру \ref env_var, либо \ref NULL, если переменная окружения не была найдена.
 */
struct env_var *getenv_var(const char *name);


/**
 * Распечатать в stdout значение переменной окруженеия в формате 'имя = значение'.
 * @param name Имя переменной окружения, которая должна быть распечатана, или \ref NULL, для печати всех переменных.
 * @return Общее количество имеющихся переменных окружения.
 */
int printenv(const char *name);

#ifdef __cplusplus
}
#endif
#endif // ifndef ENV_VARS_H_
