/**
 *  @file avi-4-streams.c
 *  @brief Одновременное воспроизведение 4-х видео 480p
 *  средствами 2D-акселератора.
 *  
 */

/*
 *      @author Соколов Сергей Валерьевич
 *      @author Ларюшин Александр Андреевич (364961@mail.ru)
 *      @author Матяш А.В. (a.matiash@set-code.ru)
 *      @copyright © ООО «Сэт Код», 2023
 */

#include <multex.h>
#include <iolib.h>
#include <tasklib.h>
#include <multimedia/a20graph.h>
#include <multimedia/avilib.h>
#include <multimedia/mpeg4codec.h>
// #include <intlib.h>

// Запуск / остановка теста
static bool continueViewing = false;

typedef struct {
    int posX;        //!< Позиция по OX
    int posY;        //!< Позиция по OY
    int subPosX;     //!< Старая позиция по OX
    int subPosY;     //!< Старая Позиция по OY
    int subSubPosX;  //!< Старая позиция по OX
    int subSubPosY;  //!< Старая позиция по OY
    int width;       //!< Ширина прямоугольника
    int height;      //!< Высота прямоугольника
    int forceX;      //!< Сила воздействия по OX
    int forceY;      //!< Сила воздействия по OY
} ddRect;

static inline void checkBorder (ddRect* ddRect1, int lftBrdr, int topBrdr, int rghtBrdr, int botBrdr) {
    // <--- - ---> Проверка границ
    if (ddRect1->posX >= rghtBrdr) {
        ddRect1->posX = rghtBrdr - (ddRect1->posX - rghtBrdr);
        ddRect1->forceX = -ddRect1->forceX;
    }
    if (ddRect1->posX <= lftBrdr) {
        ddRect1->posX = lftBrdr - (ddRect1->posX - lftBrdr);
        ddRect1->forceX = -ddRect1->forceX;
    }
    if (ddRect1->posY >= botBrdr) {
        ddRect1->posY = botBrdr - (ddRect1->posY - botBrdr);
        ddRect1->forceY = -ddRect1->forceY;
    }
    if (ddRect1->posY <= topBrdr) {
        ddRect1->posY = topBrdr - (ddRect1->posY - topBrdr);
        ddRect1->forceY = -ddRect1->forceY;
    }
}

typedef struct {
    int x;
    int y;
} ddPoint;

static inline void getCopyBgPosX (ddRect ddRect1, ddPoint* p1, ddPoint* p2, ddPoint* p3, ddPoint* p4) {
    // <--- - ---> Определяем блок для копирования координаты OX 4-х точек
    int dx = ddRect1.posX - ddRect1.subSubPosX;
    if (dx > 0) {
        // Вправо
        p1->x = ddRect1.subSubPosX;
        p2->x = ddRect1.posX;
        p3->x = p2->x;
        p4->x = ddRect1.subSubPosX + ddRect1.width;
        return;
    }
    if (dx < 0) {
        // Влево
        p1->x = ddRect1.subSubPosX + ddRect1.width + dx;
        p2->x = ddRect1.subSubPosX + ddRect1.width;
        p3->x = ddRect1.subSubPosX;
        p4->x = ddRect1.subSubPosX + ddRect1.width + dx;
        return;
    }
    // Вертикальный "отскок"
    p3->x = ddRect1.posX;
    p4->x = ddRect1.posX + ddRect1.width;
    p1->x = p2->x = p3->x;
    // printf ("dx == 0\n");
}

static inline void getCopyBgPosY (ddRect ddRect1, ddPoint* p1, ddPoint* p2, ddPoint* p3, ddPoint* p4) {
    // <--- - ---> Определяем блок для копирования координаты OY 4-х точек
    int dy = ddRect1.posY - ddRect1.subSubPosY;
    if (dy > 0) {
        // Вниз
        p1->y = ddRect1.subSubPosY;
        p2->y = ddRect1.subSubPosY + ddRect1.height;
        p3->y = p1->y;
        p4->y = ddRect1.subSubPosY + dy;
        return;
    }
    if (dy < 0) {
        // Вверх
        p1->y = ddRect1.subSubPosY;
        p2->y = ddRect1.subSubPosY + ddRect1.height;
        p3->y = ddRect1.subSubPosY + ddRect1.height + dy;
        p4->y = p2->y;
        return;
    }
    // Горизонтальный "отскок"
    p1->y = ddRect1.posY;
    p2->y = p1->y + ddRect1.height;
    p3->y = p4->y = p2->y;
}

static inline void copyChangeRegion (HDC* pDst, HDC* pSrc, ddPoint p1, ddPoint p2, int* cpWidth, int* cpHeight) {
    // <--- - ---> Скопировать область
    *cpWidth = p1.x - p2.x;
    *cpHeight = p1.y - p2.y;
    if (*cpWidth < 0) {
        *cpWidth = -(*cpWidth);
    }
    if (*cpHeight < 0) {
        *cpHeight = -(*cpHeight);
    }
    // printf ("p1.x = %i, p1.y = %i, wid = %i, hei = %i\n", p1.x, p1.y, *cpWidth, *cpHeight);
    if ((*cpWidth > 0) && (*cpHeight > 0)) {
        bitBlt ((HDC)pDst, p1.x, p1.y, *cpWidth, *cpHeight, (HDC)pSrc, p1.x, p1.y, 0, 0, 0);
    }
    // else {
    //  printf ("LINE %i\n", __LINE__);
    // }
}

static int rDg9 () {

    if (!getLFB ())
        setVideoMode (MODE_1920x1080, 32);
    init_2D_engine ();

    // Размеры видео в файлах
    int width1, height1;
    int width2, height2;
    int width3, height3;
    int width4, height4;
    // Длины считанных кадров
    int len1, len2, len3, len4;
    // Количество кадров в каждом видео
    int frames1, frames2, frames3, frames4;

    // Параметры движения окон
    const int maxForceX = 16, maxForceY = 16;
    const int offsetX = 1, offsetY = 1;

    // Поверхность - подложка
    HDC surfBackground = loadPNGSurface ("surf/wall.png");

    // Дескрипторы открытых файлов
    AVI_HANDLE ah1 = openAVIFile ("avi/480p/sky.avi",       &width1, &height1, &frames1);
    AVI_HANDLE ah2 = openAVIFile ("avi/480p/film.avi",      &width2, &height2, &frames2);
    AVI_HANDLE ah3 = openAVIFile ("avi/480p/hearts.avi",    &width3, &height3, &frames3);
    AVI_HANDLE ah4 = openAVIFile ("avi/480p/countdwn.avi",  &width4, &height4, &frames4);


    // Размеры выходного фрейма для каждого видео
    int vidOutWidth = 854, vidOutHeight = 480;

    // Настройка координат запуска
    static ddRect ddRect1, ddRect2, ddRect3, ddRect4;

    ddRect1.posX = ((Display.Width - vidOutWidth) / 3) * 0;
    ddRect1.posY = ((Display.Height - vidOutHeight) / 3) * 0;
    ddRect1.subPosX = ddRect1.posX;
    ddRect1.subPosY = ddRect1.posY;
    ddRect1.subSubPosX = ddRect1.subPosX;
    ddRect1.subSubPosY = ddRect1.subPosY;

    ddRect2.posX = ((Display.Width - vidOutWidth) / 3) * 1;
    ddRect2.posY = ((Display.Height - vidOutHeight) / 3) * 1;
    ddRect2.subPosX = ddRect2.posX;
    ddRect2.subPosY = ddRect2.posY;
    ddRect2.subSubPosX = ddRect2.subPosX;
    ddRect2.subSubPosY = ddRect2.subPosY;

    ddRect3.posX = ((Display.Width - vidOutWidth) / 3) * 2;
    ddRect3.posY = ((Display.Height - vidOutHeight) / 3) * 2;
    ddRect3.subPosX = ddRect3.posX;
    ddRect3.subPosY = ddRect3.posY;
    ddRect3.subSubPosX = ddRect3.subPosX;
    ddRect3.subSubPosY = ddRect3.subPosY;

    ddRect4.posX = ((Display.Width - vidOutWidth) / 3) * 3;
    ddRect4.posY = ((Display.Height - vidOutHeight) / 3) * 3;
    ddRect4.subPosX = ddRect4.posX;
    ddRect4.subPosY = ddRect4.posY;
    ddRect4.subSubPosX = ddRect4.subPosX;
    ddRect4.subSubPosY = ddRect4.subPosY;


    // Экран 1
    ddRect1.forceX = (rand () % maxForceX) + offsetX;
    ddRect1.forceY = (rand () % maxForceY) + offsetY;
    ddRect1.width = vidOutWidth;
    ddRect1.height = vidOutHeight;

    // Экран 2
    ddRect2.forceX = (rand () % maxForceX) + offsetX;
    ddRect2.forceY = (rand () % maxForceY) + offsetY;
    ddRect2.width = vidOutWidth;
    ddRect2.height = vidOutHeight;

    // Экран 3
    ddRect3.forceX = (rand () % maxForceX) + offsetX;
    ddRect3.forceY = (rand () % maxForceY) + offsetY;
    ddRect3.width = vidOutWidth;
    ddRect3.height = vidOutHeight;

    // Экран 4
    ddRect4.forceX = (rand () % maxForceX) + offsetX;
    ddRect4.forceY = (rand () % maxForceY) + offsetY;
    ddRect4.width = vidOutWidth;
    ddRect4.height = vidOutHeight;

    int dd1 = mpeg4InitDecoder (width1, height1);
    int dd2 = mpeg4InitDecoder (width2, height2);
    int dd3 = mpeg4InitDecoder (width3, height3);
    int dd4 = mpeg4InitDecoder (width4, height4);

    if ((!dd1) || (!dd2) || (!dd3) || (!dd4)) {
        printf ("line %i\n", __LINE__);
        return ERROR;
    }

    unsigned char* stream1 = getMpeg4InptFrameBuff (dd1);
    unsigned char* stream2 = getMpeg4InptFrameBuff (dd2);
    unsigned char* stream3 = getMpeg4InptFrameBuff (dd3);
    unsigned char* stream4 = getMpeg4InptFrameBuff (dd4);

    HDC* pScreen = (HDC*)getScreenSurface ();
    HDC* pConstr = (HDC*)getConstrSurface ();

    int screenWdth = getScreenSurface ()->w;
    int screenHght = getScreenSurface ()->h;

    int lftBrdr1 = 0, topBrdr1 = 0, rghtBrdr1 = screenWdth - vidOutWidth, botBrdr1 = screenHght - vidOutHeight;
    int lftBrdr2 = 0, topBrdr2 = 0, rghtBrdr2 = screenWdth - vidOutWidth, botBrdr2 = screenHght - vidOutHeight;
    int lftBrdr3 = 0, topBrdr3 = 0, rghtBrdr3 = screenWdth - vidOutWidth, botBrdr3 = screenHght - vidOutHeight;
    int lftBrdr4 = 0, topBrdr4 = 0, rghtBrdr4 = screenWdth - vidOutWidth, botBrdr4 = screenHght - vidOutHeight;

    // Поверхности, содержащие декодированный кадр
    /// [avi surface newsurf]
    HDC pSrf0 = newSurface (width1, height1, G2D_FMT_PYUV420UVC);
    /// [avi surface newsurf]
    HDC pSrf1 = newSurface (width2, height2, G2D_FMT_PYUV420UVC);
    HDC pSrf2 = newSurface (width3, height3, G2D_FMT_PYUV420UVC);
    HDC pSrf3 = newSurface (width4, height4, G2D_FMT_PYUV420UVC);

    HDC* pTmpSrf = (HDC*)newSurface (screenWdth, screenHght, G2D_FMT_RGB565);
    stretchBlt ((HDC)pTmpSrf, 0, 0, Display.Width, Display.Height, surfBackground, 1, 1, surfBackground->w - 2, surfBackground->h - 2, 0, 0, G2D_BLT_NONE);

    bitBlt ((HDC)pConstr, 0, 0, screenWdth, screenHght, (HDC)pTmpSrf, 0, 0, 0, 0, 0);
    bitBlt ((HDC)pScreen, 0, 0, screenWdth, screenHght, (HDC)pTmpSrf, 0, 0, 0, 0, 0);

    do {
        /// [avi surface read]
        while ((len1 = readAVIFrame (ah1, (char*)stream1)) == -1) {
            seekToFirstVideoFrame (ah1);
        }
        /// [avi surface read]
        while ((len2 = readAVIFrame (ah2, (char*)stream2)) == -1) {
            seekToFirstVideoFrame (ah2);
            // printf ("In line %i\n", __LINE__);
        }
        while ((len3 = readAVIFrame (ah3, (char*)stream3)) == -1) {
            seekToFirstVideoFrame (ah3);
            // printf ("In line %i\n", __LINE__);
        }
        while ((len4 = readAVIFrame (ah4, (char*)stream4)) == -1) {
            seekToFirstVideoFrame (ah4);
            // printf ("In line %i\n", __LINE__);
        }

        /// [avi surface decode]
        if (mpeg4DecodeBlock (dd1, (unsigned char *)pSrf0->addr[0], len1) != OK) {
            printf ("decode error in line %i\n", __LINE__);
            return ERROR;
        }
        /// [avi surface decode]
        if (mpeg4DecodeBlock (dd2, (unsigned char *)pSrf1->addr[0], len2) != OK) {
            printf ("decode error in line %i\n", __LINE__);
            return ERROR;
        }
        if (mpeg4DecodeBlock (dd3, (unsigned char *)pSrf2->addr[0], len3) != OK) {
            printf ("decode error in line %i\n", __LINE__);
            return ERROR;
        }
        if (mpeg4DecodeBlock (dd4, (unsigned char *)pSrf3->addr[0], len4) != OK) {
            printf ("decode error in line %i\n", __LINE__);
            return ERROR;
        }

        static int cpWidth = 0, cpHeight = 0;
        static ddPoint p1, p2, p3, p4;

        getCopyBgPosX (ddRect1, &p1, &p2, &p3, &p4);
        getCopyBgPosY (ddRect1, &p1, &p2, &p3, &p4);
        copyChangeRegion (pConstr, pTmpSrf, p1, p2, &cpWidth, &cpHeight);
        copyChangeRegion (pConstr, pTmpSrf, p3, p4, &cpWidth, &cpHeight);

        getCopyBgPosX (ddRect2, &p1, &p2, &p3, &p4);
        getCopyBgPosY (ddRect2, &p1, &p2, &p3, &p4);

        copyChangeRegion (pConstr, pTmpSrf, p1, p2, &cpWidth, &cpHeight);
        copyChangeRegion (pConstr, pTmpSrf, p3, p4, &cpWidth, &cpHeight);

        getCopyBgPosX (ddRect3, &p1, &p2, &p3, &p4);
        getCopyBgPosY (ddRect3, &p1, &p2, &p3, &p4);
        copyChangeRegion (pConstr, pTmpSrf, p1, p2, &cpWidth, &cpHeight);
        copyChangeRegion (pConstr, pTmpSrf, p3, p4, &cpWidth, &cpHeight);

        getCopyBgPosX (ddRect4, &p1, &p2, &p3, &p4);
        getCopyBgPosY (ddRect4, &p1, &p2, &p3, &p4);
        copyChangeRegion (pConstr, pTmpSrf, p1, p2, &cpWidth, &cpHeight);
        copyChangeRegion (pConstr, pTmpSrf, p3, p4, &cpWidth, &cpHeight);

        /// [avi surface stretch]
        stretchBlt ((HDC)pConstr, ddRect1.posX, ddRect1.posY, vidOutWidth, vidOutHeight, (HDC)pSrf0, 0, 0, width1, height1, 0, 0, G2D_BLT_NONE);
        /// [avi surface stretch]
        stretchBlt ((HDC)pConstr, ddRect2.posX, ddRect2.posY, vidOutWidth, vidOutHeight, (HDC)pSrf1, 0, 0, width2, height2, 0, 0, G2D_BLT_NONE);
        stretchBlt ((HDC)pConstr, ddRect3.posX, ddRect3.posY, vidOutWidth, vidOutHeight, (HDC)pSrf2, 0, 0, width3, height3, 0, 0, G2D_BLT_NONE);
        stretchBlt ((HDC)pConstr, ddRect4.posX, ddRect4.posY, vidOutWidth, vidOutHeight, (HDC)pSrf3, 0, 0, width4, height4, 0, 0, G2D_BLT_NONE);


        // Переключить поверхности
        /// [avi surface flip]
        FlipScreenAndConstr ();
        waitVerticalRetrace ();
        /// [avi surface flip]

        pScreen = (HDC*)getScreenSurface ();
        pConstr = (HDC*)getConstrSurface ();

        ddRect1.subSubPosX = ddRect1.subPosX;
        ddRect1.subSubPosY = ddRect1.subPosY;
        ddRect1.subPosX = ddRect1.posX;
        ddRect1.subPosY = ddRect1.posY;
        ddRect2.subSubPosX = ddRect2.subPosX;
        ddRect2.subSubPosY = ddRect2.subPosY;
        ddRect2.subPosX = ddRect2.posX;
        ddRect2.subPosY = ddRect2.posY;
        ddRect3.subSubPosX = ddRect3.subPosX;
        ddRect3.subSubPosY = ddRect3.subPosY;
        ddRect3.subPosX = ddRect3.posX;
        ddRect3.subPosY = ddRect3.posY;
        ddRect4.subSubPosX = ddRect4.subPosX;
        ddRect4.subSubPosY = ddRect4.subPosY;
        ddRect4.subPosX = ddRect4.posX;
        ddRect4.subPosY = ddRect4.posY;

        ddRect1.posX += ddRect1.forceX;
        ddRect1.posY += ddRect1.forceY;
        ddRect2.posX += ddRect2.forceX;
        ddRect2.posY += ddRect2.forceY;
        ddRect3.posX += ddRect3.forceX;
        ddRect3.posY += ddRect3.forceY;
        ddRect4.posX += ddRect4.forceX;
        ddRect4.posY += ddRect4.forceY;

        checkBorder (&ddRect1, lftBrdr1, topBrdr1, rghtBrdr1, botBrdr1);
        checkBorder (&ddRect2, lftBrdr2, topBrdr2, rghtBrdr2, botBrdr2);
        checkBorder (&ddRect3, lftBrdr3, topBrdr3, rghtBrdr3, botBrdr3);
        checkBorder (&ddRect4, lftBrdr4, topBrdr4, rghtBrdr4, botBrdr4);

        taskDelay (1);
    } while (continueViewing);

    // Закрыть всё и выйти

    // Задний фон - оставить
    bitBlt ((HDC)pConstr, 0, 0, screenWdth, screenHght, (HDC)pTmpSrf, 0, 0, 0, 0, 0);
    bitBlt ((HDC)pScreen, 0, 0, screenWdth, screenHght, (HDC)pTmpSrf, 0, 0, 0, 0, 0);

    // Освобождение памяти
    freeMpeg4FrameMem (dd1);
    freeMpeg4FrameMem (dd2);
    freeMpeg4FrameMem (dd3);
    freeMpeg4FrameMem (dd4);

    deleteSurface ((surface_t*)pTmpSrf);
    deleteSurface (surfBackground);
    
    deleteSurface (pSrf0);
    deleteSurface (pSrf1);
    deleteSurface (pSrf2);
    deleteSurface (pSrf3);

    closeAVIFile (ah1);
    closeAVIFile (ah2);
    closeAVIFile (ah3);
    closeAVIFile (ah4);

    return OK;
}


void aviTest4 () {
    if (continueViewing) {
        return;
    }
    continueViewing = true;
    taskSpawn ("avi4Test", 10, 0, 0, (FUNCPTR)rDg9, 0);
}

void aviTest4Stop () {
    continueViewing = false;
}

