/*
Header file: hcd.h
  Host controller device structure
*/
#ifndef HCD_H_
#define HCD_H_

#ifdef __cplusplus
extern "C" {
#endif
/* Public declarations */
#include <semlib.h>

#include <hcd/ehci-hcd.h>
// #include "usb.h"
struct devrequest;

/**
 *  @brief Поддерживаемые архитектуры процессоров.
 * 
 */
typedef enum {
    usbArchUnknown, ///< Архитектура не определена.
    usbbArchNotUsed,///< Модуль USB не используется, либо не используется USB с номером \b devnum.
    usbArchA20,     ///< Процессор A20.
    usbArchV3s,     ///< Процессор V3s.
} eUsbArch;
struct ehci_hcd {
    int (*submit_control_msg) (struct usb_device *dev,
                               unsigned long pipe, void *buffer, int length, struct devrequest *setup);
    int (*submit_bulk_msg) (struct usb_device *dev,
                            unsigned long pipe, void *buffer, int length, int timeout);
    int (*submit_int_msg) (struct usb_device *dev,
                           unsigned long pipe, void *buffer, int length, int interval);
    char *TD_AREA;  // Выравненная область памяти для дескрипторов
    struct QH *qh_mark;
    struct QH *qh_work;
    struct QH *qh_list;
    struct qTD *td_work;
    unsigned int *pf_list;
    struct QH *free_qh;
    struct qTD *free_td;
    SEM_ID ehci_irq_sem;
    SEM_ID mutex;
    int rootdev;
    int devnum;
    volatile struct ehci_hccr *hccr;
    volatile struct ehci_hcor *hcor;
    unsigned int dev_bitmap;
    int dev_count;
    int irq;
    struct ehci_irq_element *ehci_irq_list;
    struct ehci_async_element *ehci_async_list;
    struct ehci_async_element *ehci_async_tail;
    int connect;
    eUsbArch arch;
};

#define EHCI_HCD(d) (struct ehci_hcd *)d->hcd;

#ifdef __cplusplus
}
#endif
#endif  // ifndef HCD_H_
