/*
     RTOS MULTEX KLOTHO

     Internet Protocol
     
     Author: Sergey V.Sokolov
     
     05.2003
 
*/
#ifndef IP_H_
#define IP_H_

#ifdef __cplusplus
extern "C" {
#endif

//#define IP_SPOIL 

#include <netdevice.h>
#include "udp.h"
/*
 * Definitions for internet protocol version 4.
 * Per RFC 791, September 1981.
 */
#define IPVERSION 4
#define SIZEOF_IPHDR 20

//#define  	REDIRECT_FEATURES // ВКлючить возможность перенаправления пакетов флагом REDIRECT_ENA
#undef  	REDIRECT_FEATURES // ОТключить возможность перенаправления пакетов флагом REDIRECT_ENA

/* Обработка фрагментированных пакетов IPv4 */
#define MAX_FIP_TOT_LEN (65536)


typedef struct ip
       {
        UINT ip_hl :4;
        UINT ip_v:4;
        UINT ip_tos:8;
        UINT ip_len  :16;
        
        UINT ip_id   :16;   // Идентификатор пакета
        UINT ip_off  :16;

#define IP_DF 0x4000   /* dont fragment flag */
#define IP_MF 0x2000   /* more fragments flag */
#define IP_OFF_MASK 0x1FFF  /* Маска выделения смещения */
        
        UINT ip_ttl  :8;
        UINT ip_p    :8;
        UINT ip_sum  :16;
        
        UINT ip_src  :32;
        
        UINT ip_dst  :32;    
       }IP_HDR;

#define IP_MAXPACKET    65535           /* maximum packet size */


/*
 * Протоколы
 */
#define IPPROTO_IP  0  /* dummy for IP */
#define IPPROTO_ICMP  1  /* control message protocol */
#define IPPROTO_GGP  3  /* gateway^2 (deprecated) */
#define IPPROTO_TCP  6  /* tcp */
#define IPPROTO_EGP  8  /* exterior gateway protocol */
#define IPPROTO_PUP  12  /* pup */
#define IPPROTO_UDP  17  /* user datagram protocol */
#define IPPROTO_IDP  22  /* xns idp */

#define IPPROTO_RAW  255  /* raw IP packet */
#define IPPROTO_MAX  256

// // Внутренняя структура MULTEX
// typedef struct ip_packet
//            {
//             UINT length;
//             UCHAR protocol;
//             ip_addr_t s_addr;
//             ip_addr_t d_addr;
//             char *data;
//            }IP_PACKET; 

typedef void (*protoTask)(int ndev, IP_PACKET *packet);

// Псевдозаголовок IP
typedef struct ip_ovl  
                { 
                 UCHAR ih_x1;   // всегда 0
                 UCHAR ih_pr;   // протокол
                 USHORT ih_len; // длина пакета 
                 ip_addr_t ih_src;
                 ip_addr_t ih_dst;
                }IP_OVL; 

extern STATUS ipProcessData(unsigned int idx, const void *data);
extern void ipInitProto();
extern void ipAddProto(UCHAR protocol, protoTask task);                
extern STATUS ipSendPacket(IP_HDR *ip);
extern IP_HDR *ipCreateBuffer(UINT t_addr, UCHAR protocol, int length);


// Informational functions. IP statistics.

// количество применений правил к айпи пакетам

unsigned ipPktTotal   (void);
unsigned ipPktValid   (void);

unsigned ipPktBadaddr (void);

unsigned ipPktInvalid (void);
unsigned ipPktBadchks (void);
unsigned ipPktBadprot (void);
unsigned ipPktBadlen  (void);
unsigned ipPktNodata  (void);

unsigned ipPktRedir   (void);
unsigned ipPktRedirs  (void);
unsigned ipPktRedird  (void);

// INVALID = BADCHKS + BADPROT + BADLEN  + NODATA
// REDIR   = REDIRS  + REDIRD
// TOTAL   = VALID   + BADADDR + INVALID + REDIR

char* ipPktStatString(void); // суммарная информация в виде текстовой строки



#ifdef __cplusplus
}
#endif
#endif // ifndef IP_H_
