/*
Header file: irq.h
*/
#ifndef IRQ_H_
#define IRQ_H_

#ifdef __cplusplus
extern "C" {
#endif

/* Public declarations */
/**
 *  @brief Информация о прерывании.
 *  @details Информация о прерывании считывается
 *  с помощью gic_irq_info() и используется только 
 *  для отладки.
 */
typedef struct {
    bool enabled;
    int groupPriority;
    int subPriority;
} tIrqInfo;


int irq_init();
int irq_enable(int vector, int group, int priority);
int irq_disable(int vector);
int irq_get_source(void);
void irq_done(int vector);

/**
 *  @name Отладка и печать параметров
 *  @{
 */

/**
 *  @brief Печать справки о зарегистрированных прерываниях.
 *  @details Вывод приоритетов активных прерываний в консоль.
 *  Используется для отладки.
 *  @note Реализация в lib_kernel.
 */
void irq ();

/**
 *  @brief Информация о настройках прерывания.
 *  @details Функция позволяет получить настройки
 *  конкретного аппаратного прерывания.
 *  
 *  @param irq Номер прерывания.
 *  @param info Считанная информация.
 */
void irq_info (int irq, tIrqInfo *info);
/// @}

/// @privatesection

int irqArchInit ();


#ifdef __cplusplus
}
#endif
#endif // ifndef IRQ_H_
