/**
    @addtogroup groupMultimedia
    @{
    @file fontsdefines.h
    @brief Различные общие структуры, перечисления и дефайны для шрифтов.
    @details Предполагается, что данный файл будет подключаться во все шрифтовые интерфейсы (ttf шрифты, спрайтовые шрифты, может еще куда-нибудь).
    @}
 */

/*
    @date Created on: 4 сент. 2019 г.
    @author Dubovich Timofey
*/


#ifndef __FONTDEFINES_H_
#define __FONTDEFINES_H_

//! @details Количество ASCII-символов, которое можно вывести на экран.
#define ASCII_PRINTED_SYMBOLS_AMOUNT ('~' - ' ' + 1)    //Все, что можно вывести, включая пробел

//! @details Проверка на то, что символ - выводимый и из ascii.
#define SYMBOL_IS_PRINTED_ASCII(x) ((x) >= ' ' && (x) <= '~')

//! @details Перевод символа (например ' ') в индекс массива (0).
#define SYMBOL_CODE_TO_ARRAY_INDEX(x) ((x) - ' ')

//! @details Перевод индекса массива (например 0) в символ (' ').
#define ARRAY_INDEX_TO_SYMBOL(x) ((x) + ' ')


/**
 * @brief Структура прямоугольника.
 */
typedef struct {
 int x; //!< @details X-координата.
 int y; //!< @details Y-координата.
 int w; //!< @details Ширина прямоугольника.
 int h; //!< @details Высота прямоугольника.
} textRect;
//! @details Переименование типа, добавлено для совместимости и улучшения читаемости.
typedef textRect sTextRect;
//! @@details Указатель на прямоугольник, добавлен для совместимости и улучшения читаемости.
typedef sTextRect* pTextRect;


/**
 * @brief Опции выравнивания шрифта при выводе в прямоугольник.
 * @details Наборы опций {alignHLeft, alignHCenter, alignHRight} и {alignVTop, alignVMiddle, alignVBottom} допускают выбор только одной опции из каждого набора.
 * Если из набора не будет выбрана ни одна из опций, то по-умолчанию будет использоваться опция noAlign.
 * Опции из разных наборов можно совмещать через оператор '|'.
 */
typedef enum {
    noAlign = 0,                //!< @details Не использовать выравнивание, текст будет выводиться в левый нижний угол прямоугольника, опция по-умолчанию.
    alignHLeft = 0x00000001,    //!< @details Выравнивать горизонтальное положение текста по левой стороне прямоугольника.
    alignHCenter = 0x00000002,  //!< @details Выравнивать горизонтальное положение текста по центру прямоугольника.
    alignHRight  = 0x00000004,  //!< @details Выравнивать горизонтальное положение текста по правой стороне прямоугольника.
    alignVTop  = 0x00000008,    //!< @details Выравнивать вертикальное положение текста по верхней стороне прямоугольника.
    alignVMiddle = 0x00000010,  //!< @details Выравнивать вертикальное положение текста по центру прямоугольника.
    alignVBottom = 0x00000020   //!< @details Выравнивать вертикальное положение текста по нижней стороне прямоугольника.
} alignType;
//! @details Тип, добавленный для совместимости и улучшения читаемости.
typedef alignType eAlignType;

#endif /* __FONTDEFINES_H_ */
