/**
 * @addtogroup groupCore
 * @{
 * @file names.h
 * @brief Функции для работы с таблицами символов.
 * @}
 */

#ifndef _NAMESINC
#define _NAMESINC

#ifdef __cplusplus
extern "C" {
#endif

#include "multex.h"
#include "stddef.h"
#include "symtbl.h"

/**
 * Найти символ в таблицах символов и вернуть связанный с ним указатель.
 * @param pName
 * @param[out] pTypeOutputVar Указатель на char-переменную, в которую будет записан тип символа.
 * @return Связанный с символом указатель или NULL, если символ не был найден.
 */
void *_findSTName(const char *pName, char *pTypeOutputVar);

/**
 * Создать новую таблицу символов.
 * @param size Максимальный размер таблицы символов.
 * @return Указатель на выделенную таблицу или \ref NULL при ошибке.
 */
dynSymTbl* createDynSymTbl(size_t size);

/**
 * Добавить символ в таблицу символов.
 * @param pTable Таблица символов.
 * @param pName Имя символа.
 * @param addr Адрес, к которому должен быть привязан символ.
 * @param type Тип символа, как у компилятора GCC ('T' для функции, 'D' для переменной).
 * @return \ref OK при успехе, \ref ERROR иначе.
 * @note Таблица имеет ограниченное кол-во "слотов" под символы.
 */
STATUS appendSymbol(dynSymTbl *pTable, const char *pName, void* addr, char type);


/**
 * Подключить дополнительную таблицу символов (можно подключить неограниченное кол-во таблиц).
 * @param pTable Выделенная и заполненная таблица символов.
 * @return \ref OK при успехе, \ref ERROR иначе.
 */
STATUS registerSymTbl(dynSymTbl *pTable);

/**
 * @private
 * @brief Подключить основную (статическую) таблицу символов.
 * @details Вызывается при старте ядра ОС.
 */
void registerMainSymTbl(void);




#ifdef __cplusplus
}
#endif

        
#endif    

