/**
 *  @addtogroup groupStdTypes
 *  @{
 *  @file stddef.h
 *  @brief Стандартные определения.
 *  @details
 *  @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.19</a>.
 *  @}
 */

/*
 *  @author Sergey Sokolov
 *  @author Dubovich Timofey
*/
#ifndef __STDDEF_H_
#define __STDDEF_H_

#include <multex.h>

//! @details Константа нулевого указателя.
#define NULL ((void*)0)

//! @details Вычисление смещение в байтах поля структуры или объединения от начала структуры или объединения.
#define offsetof(TYPE, MEMBER) __builtin_offsetof (TYPE, MEMBER)

//! @details Тип, представляющий результат операции вычитания над двумя указателями.
typedef int ptrdiff_t;

//! @details Тип, представляющий возврат оператора \b sizeof.
//! Определён как <b>__typeof(sizeof(0))</b>.
typedef __typeof(sizeof(0)) size_t;

//! @details Тип, представляющий широкий символ.
typedef int wchar_t;

#if __STDC_VERSION__  >= 201112L
    //! Тип с максимально большим поддерживаемым выравниванием.
    typedef long double max_align_t;
#endif


#endif /* __STDDEF_H_ */
