/**
 * @addtogroup groupCore
 * @{
 * @file symTbl.h
 * @brief Структуры таблиц символов и объявление статической таблицы символов.
 * @}
 */

#ifndef _SYMTBLINC
#define _SYMTBLINC

#ifdef __cplusplus
extern "C" {
#endif

#include "stddef.h"

/**
 * Символ и ассоциируемый с ним адрес.
 */
typedef struct
{
    const char *name;   //!< Имя символа.
    void *ptr;          //!< Адрес.
    char typ;           //!< Тип символа.
} SYMBOL;

/**
 * Таблица символов.
 */
typedef struct DYNSYMTBL
{
    size_t size;                //!< Максимальное кол-во символов в таблице.
    size_t ptr;                 //!< Фактическое кол-во символов в таблице.
    SYMBOL *data;               //!< Выделенный буфер под символы размером size.
    struct DYNSYMTBL *next;     //!< Указатель для обслуживания списка.
} dynSymTbl;
                  

// Статическая таблица символов.
// Фактически таблица генерируется перед линковкой итогового бинарника и включает в себя все символы из
// фактически использованных объектных файлов, либо символы из файлов, явно заданных для линковки.
// Альтернативно, на этапе линковки возможно вместо настоящей таблицы символов (которую использует линковщик)
// подключить пустую фальшивую таблицу, для отключения возможности динамического поиска символов.
extern size_t symTblSize; //! Размер статической таблицы символов.
extern SYMBOL symTbl[];   //! Статическая таблица символов (размером symTblSize).

#ifdef __cplusplus
}
#endif

        
#endif         
