/*
 * Несмотря на название, не имеет никакого отношения к TCP
 * Таймер, вызывающий обработчик раз в некоторый промежуток времени
 * Header file: Timer.h
 */
#ifndef TCP_TIMER_H_
#define TCP_TIMER_H_

#ifdef __cplusplus
extern "C" {
#endif


#include <tasklib.h>
#include <semlib.h>


/* Public declarations */
#define TIMER_SIGNATURE (0xC12E7821)   // Уникальный идентификатор таймера
#define TIMER_MAX_NAME_LENGTH (10)     //Имя вида TMR_'Имя', для поля 'Имя' максимальный размер TIMER_MAX_NAME_LENGTH

typedef struct timer_id {
         UINT signature; // Сигнатура таймера
         UINT interval;  // Интервал таймера
         TASK_ID thread; // Задача таймера
         SEM_ID semT;    // Семафор таймера
         SEM_ID semC;    // Семафор завершения
         void *ovner;    // Владелец таймера
         int  f_cancel;  // Флаг завершения
         char timer_name[TIMER_MAX_NAME_LENGTH+5];  //5 байт на TMR_ и нулевой символ
         int (*onTimer)(struct timer_id *tmr);
        }TIMER_ID;
        
// Создает таймер с периодом interval (измеряется в тиках таймера)
// и подключает обработчик onTimer
// В списке процессов таймер имеет имя вида "TMR_name" или "TMR", если name = 0
// Если обработчик возвращает не 0, то таймер останавливается
TIMER_ID *createTimer(void *ovner,int interval,int (*onTimer)(TIMER_ID *tmr),char* name);
// Перезапускает таймер
STATUS restartTimer(TIMER_ID *tmr);
// Удаляет таймер
STATUS deleteTimer(TIMER_ID *tmr);

#ifdef __cplusplus
}
#endif
#endif // ifndef TIMER_H_
