/**
    @addtogroup groupCore 
    @{
    @file timer.h
    @brief Управление системным таймером.
    @details 
    @see Подробнее о системном таймере см. в главе \ref sectionSysTimer.
    @}
    

*/

#ifndef TIMER_H_
#define TIMER_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned long tick_t;

/**
 *  @brief Задать частоту системного таймера.
 *  @details Функция устанавливает заданную частоту системного таймера.
 *  
 *  @param ticks Частота в Герцах, на которую будет перестроен таймер.
 *  @return Функция возвращает \b OK.
 */
int sysClkRateSet (int ticks);

/**
 *  @brief Количество тиков таймера в секунду.
 *  @details Функция возвращает количество тиков системного таймера
 *  за одну секунду. Функция не производит физических замеров частоты таймера,
 *  а лишь опрашивает внутреннюю переменную, поэтому выполняется быстро.
 *
 *  @return Количество тиков системного таймера
 *  за одну секунду.
 */
int sysClkRateGet ();

/**
 *  @brief Число тиков таймера с момента включения.
 *  @details Функция возвращает число тиков таймера 
 *  с момента включения системы до момента ее вызова. 
 *  Счетчик тиков помещается во внутреннюю 32-битную переменную 
 *  и может быть изменен пользовательской задачей с помощью вызова 
 *  функции tickSet(). В этом случае возвращаемое функцией значение 
 *  будет определяться интервалом между записью счетчика и ее вызовом,
 *  а также записываемой величиной.
 *  
 *  @return Количество тиков таймера.
 */
unsigned long tickGet ();

/**
 *  @brief Задать значение системного таймера.
 *  @details Функция устанавливает значение счетчика тиков системного таймера.
 *  
 *  @param val Записываемое в счетчик значение.
 */
void tickSet (unsigned long val);

/**
 *  @brief Аппаратная перезагрузка.
 *  @details Перезагрузка процессора с помощью аппаратного
 *  watchdog таймера через 0,5с.
 *  
 */
void hard_reset ();


/// @privatesection
int init_hw_timer ();
void assignTickAnnounce (void (*ta) (void));
// int start_hw_timer ();
// void stop_hw_timer ();

#ifdef __cplusplus
}
#endif
#endif // ifndef TIMER_H_
