/*
Header file: txtarea.h
*/
#ifndef TXTAREA_H_
#define TXTAREA_H_

#ifdef __cplusplus
extern "C" {
#endif
/* Public declarations */
#define TXTA_SIGNATURE 0xba68c529

#define TXTA_FROMBEG 1
#define TXTA_FROMCUR 2
#define TXTA_FROMEND 3
#define TXTA_UP 	 4
#define TXTA_DN 	 5
#define TXTA_IDSPACE 0

typedef struct txtarea_type
{
  unsigned signature; 	   // сигнатура
  unsigned erutangis;	   // инвертированная она же
  unsigned iter;		   // количество переиницализаций
  
  char name[16];		   // имя лога
  int size;				   // размер области данных
  int idcur;			   // сквозной номер
  int stopped;			   // запись остановлена
  int autostop;			   // обратный счетчик для автоостановки 
  
  int first;			   // позиция старейшей записи
  int pos;				   // позиция новейшей записи

  int cfirst;			   // (Копия) позиция старейшей записи
  int cpos;				   // (Копия) позиция новейшей записи

  int *vfirst;			   // (Просмотр) позиция старейшей записи
  int *vpos;			   // (Просмотр) позиция новейшей записи

  int lastONtop;		   // последние строки сверху 
  int winSz;			   // размер окна просмотра
  int winPos;			   // позиция окна просмотра (сквозной номер)

  int  ftpaCur;			   // Текущая запись в locked для реализации
//  char ftpaBuf[256];	   // Вспомогательный буфер для ftpa read   
//  char ftpaPos;	   		   // Позиция во вспомогательном буфере
     
//  int winSrc;			   // Основное или копия
  struct txtarea_type *next; // Для объединения в список.
  unsigned char *view;		 // Указатель на просматриваемые данные
  unsigned char *copy;		 // Копия данных
  unsigned char data[0];	 // Непосредственно данные
}TTxta;
typedef TTxta *pTxta;

// (Служебноотладочное!) Проверка и создание семафора разделения доступа к тхтам
void txtaSemPrimary(void);

// Создать тхта с именем name в хипе с помощью "чистого" маллока и "чистого" сембкриэйт
pTxta txtaCreate(int size, char *name); 

// Создание тхта с именем name в указанной области памяти buf размера bufsize
// Если там уже находитя тхта (прописаны сигнатуры), то инициализироваться не будет 
pTxta txtaReRegister(void *buf, int bufsize, char *name);

// Выделение памяти с помощью alloc, создание тхта размера size с именем name в этой области памяти buf 
// Если там уже находитя тхта (прописаны сигнатуры), то инициализироваться не будет 
pTxta txtaReCreate( void*(*alloc)(int) , int size, char *name );

// Проверка txta на корректность
int txtaCheckSign(pTxta A);


// Вывести системную таблицу тхтей
void txtaPrintList(void);

// Добавление строки к тхте
void taprintf(pTxta area, char* s, ...);   // без квитирования
void tadtprintf(pTxta area, char* s, ...); // дата-время
void tatprintf(pTxta area, char* s, ...);  // время
void tagprintf(pTxta area, char* s, ...);  // tickget as HEX
void tatgprintf(pTxta area, char* s, ...); // tickget as time

// Добавить строку завершённую дампом (старший полубайт вперёд, байты слитно)
void tagprinthex(pTxta area, void* dump, int len, char* s, ...); // tickget as HEX

void txtaPrintAll(pTxta A); // распечатать _ВСЕ_ содержимое
//int txtaPrintRep(pTxta A);  // повторить последнюю из сделанных распечаток
// Выборочная печать, num количество, reverse - обратный порядок, from - смещение
//int txtaPrintLast(pTxta A, int num, int from, int reverse); // смещение от конца назад
//int txtaPrintFirst(pTxta A, int num, int from, int reverse); // смещение от начала вперед

pTxta txtaCurByAddr(pTxta A);            //Установить тхту с идентификатором A как текущую
pTxta txtaCurByName(char *name); 		 //Установить тхту с именем name как текущую
pTxta txtaCurByNum(int num);	 		 //Установить тхту с номером num в системной таблице как текущую
//int txtaPrintCRep(void);		 	     // Повторить последнюю распечатку для текущей тхты
//int txtaPrintCLast(int num, int from); // Выборочная распечатка с конца для текущей тхты
//int txtaPrintCFirst(int num, int from);// Выборочная распечатка с начала для текущей тхты
void txtaSetReverse(int m);	 	 		 // Вкл/выкл обратного порядка строк

int txtaPrintWin(pTxta A, int shift, int from); // Распечатка окна, shift - смещение, from - откуда
void txtaSetWinRev(pTxta A, int mode);	 		// Вкл/выкл обратного порядка строк
void txtaSetWinSz(pTxta A, int num);			// Устанока размера окна распечатккки
void txtaSaveToFile(pTxta A, char *name);		// Сбросить все содержимое в файл с именем name
void txtaSaveLocked(pTxta A, char *name);		// Сбросить предварительно сделанный снимок в файл

int txtaStop(pTxta A);	  	 	  	  // Остановить записть в тхту A
int txtaStart(pTxta A);				  // Запустить записть в тхту A
int txtaClear(pTxta A);				  // Очистить содержимое тхты А
void txtaAutostop(pTxta A, int num);  // Сбросить все содержимое в файл с именем name


int txtaPrintCWin(int shift, int from);	 // Распечатка окна для текущей тхты

void txtaLock(pTxta A);		 	 		 // Создание снимка тхты во встроенный дублирующий буфер
void txtaSource(pTxta A, int src);		 // Выбор рабочего буфера: 1 - живой, 2 - замороженный 
void txtaCLock(void); 	 	 			 // (для текущей) Создание снимка текущей тхты во встроенный дублирующий буфер
void txtaCSource(int src);	 			 // (для текущей) Выбор рабочего буфера : 1 - живой, 2 - замороженный 
void txtaCSetWinSz(int num);			 // (для текущей) Устанока размера окна распечатккки
void txtaCSetWinRev(int mode);			 // (для текущей) Вкл/выкл обратного порядка строк 
void txtaCStop(void);					 // (для текущей) Остановить записть в тхту A
void txtaCStart(void);					 // (для текущей) Запустить записть в тхту A
void txtaCClear(void);					 // (для текущей) Очистить содержимое тхты А
void txtaCSaveToFile(char *name);		 // (для текущей) Сбросить все содержимое в файл с именем name
void txtaCSaveLocked(char *name);		 // (для текущей) Сбросить предварительно сделанный снимок в файл
void txtaCAutostop(int num);			 // (для текущей) Сбросить все содержимое в файл с именем name

pTxta txtaCurrent(void);				 // Возвращает идентификатор текущей тхты (адрес в памяти)
void txtaScreenShadow(int mode);		 // Режим теневой печати на экран вкл/выкл

// Макросы для создания ftpa из txta
int ftpaTxtaRead(int iA, void *buf, int size);
int ftpaTxtaClopen(int iA, int mode);



//***********************************************************************
//                          "Новый" интерфейс
//***********************************************************************
#include "multex.h"
typedef pTxta pLog;

//Создать лог
pLog log_Create(char* log_name, unsigned int log_size);

//Записать в лог запись с квитированием временем через тики процессора
STATUS log_Printf(pLog log, char* s, ...) __attribute__ ((format (printf, 2, 3)));


#ifdef __cplusplus
}
#endif
#endif // ifndef TXTAREA_H_
